/*****************************************************************************
 *   timer.c:  Timer C file for NXP LPC230x Family Microprocessors
 *
 *   Copyright(C) 2006, NXP Semiconductor
 *   All rights reserved.
 *
 *   History
 *   2006.09.01  ver 1.00    Prelimnary version, first Release
 *
******************************************************************************/
#include <nxp/iolpc1768.h>
#include "type.h"
#include "irq.h"
#include "timer.h"

#include <intrinsics.h>

volatile DWORD timer_counter = 0;

/******************************************************************************
** Function name:   Timer0Handler
**
** Descriptions:    Timer/Counter 0 interrupt handler
**        executes each 10ms @ 60 MHz CPU Clock
**
** parameters:      None
** Returned value:    None
**
******************************************************************************/
void TMR0_IRQHandler (void)
{
  T0IR = 1;     /* clear interrupt flag */

  __enable_interrupt();       /* handles nested interrupt */

  timer_counter++;
  /**/
  NVIC_ClrPend(NVIC_TIMER0);
}

/******************************************************************************
** Function name:   enable_timer
**
** Descriptions:    Enable timer
**
** parameters:      timer number: 0 or 1
** Returned value:    None
**
******************************************************************************/
void enable_timer( BYTE timer_num )
{
  if ( timer_num == 0 )
  {
    T0TCR = 1;
  }
  else
  {
    T1TCR = 1;
  }
  return;
}

/******************************************************************************
** Function name:   disable_timer
**
** Descriptions:    Disable timer
**
** parameters:      timer number: 0 or 1
** Returned value:    None
**
******************************************************************************/
void disable_timer( BYTE timer_num )
{
  if ( timer_num == 0 )
  {
    T0TCR = 0;
  }
  else
  {
    T1TCR = 0;
  }
  return;
}

/******************************************************************************
** Function name:   reset_timer
**
** Descriptions:    Reset timer
**
** parameters:      timer number: 0 or 1
** Returned value:    None
**
******************************************************************************/
void reset_timer( BYTE timer_num )
{
  DWORD regVal;

  if ( timer_num == 0 )
  {
    regVal = T0TCR;
    regVal |= 0x02;
    T0TCR = regVal;
  }
  else
  {
    regVal = T1TCR;
    regVal |= 0x02;
    T1TCR = regVal;
  }
  return;
}

/******************************************************************************
** Function name:   init_timer
**
** Descriptions:    Initialize timer, set timer interval, reset timer,
**            install timer interrupt handler
**
** parameters:      None
** Returned value:    true or false, if the interrupt handler can't be
**            installed, return false.
**
******************************************************************************/
DWORD init_timer ( DWORD TimerInterval )
{
  timer_counter = 0;
  T0MR0 = TimerInterval;
  T0MCR = 3;        /* Interrupt and Reset on MR0 */
  
  NVIC_IntEnable(NVIC_TIMER0);
  NVIC_IntPri(NVIC_TIMER0, HIGHEST_PRIORITY);
  
  return (TRUE);
}

/******************************************************************************
**                            End Of File
******************************************************************************/
