/*****************************************************************************
 *   extint.c:  External interrupt API C file for NXP LPC17xx
 *   Family Microprocessors
 *
 *   Copyright(C) 2006, NXP Semiconductor
 *   All rights reserved.
 *
 *   History
 *   2006.07.13  ver 1.00    Prelimnary version, first Release
 *
 *****************************************************************************/
#include <nxp/iolpc1768.h>
#include "type.h"
#include "irq.h"
#include "extint.h"
#include "board.h"

#include <intrinsics.h>

volatile DWORD eint0_counter;

/*****************************************************************************
** Function name:   EINT3_IRQHandler
**
** Descriptions:    external INT handler
**
** parameters:      None
** Returned value:    None
**
*****************************************************************************/
void EINT3_IRQHandler (void)
{
  IO0INTCLR = B1_MASK;        /* clear interrupt */

  __enable_interrupt();       /* handles nested interrupt */

  eint0_counter++;

  /* flashing USB Link LED */
  LED1_FIO ^= LED1_MASK;

  NVIC_ClrPend(NVIC_EINT3);
}

/*****************************************************************************
** Function name:   EINTInit
**
** Descriptions:    Initialize external interrupt pin and
**        install interrupt handler
**
** parameters:      None
** Returned value:    true or false, return false if the interrupt
**        handler can't be installed to the VIC table.
**
*****************************************************************************/
DWORD EINTInit( void )
{

  IO0INTENF = B1_MASK;        /* B1 interrupt by falling edge. */

  NVIC_IntEnable(NVIC_EINT3);
  NVIC_IntPri(NVIC_EINT3,HIGHEST_PRIORITY);
  
  return( TRUE );
}

/******************************************************************************
**                            End Of File
******************************************************************************/
