/*****************************************************************************
 *   rtctest.c:  main C entry file for NXP LPC23xx/24xx Family Microprocessors
 *
 *   Copyright(C) 2006, NXP Semiconductor
 *   All rights reserved.
 *
 *   History
 *   2006.07.13  ver 1.00    Prelimnary version, first Release
 *
 * Note:
 *  After power-up the controller get clock from internal RC oscillator that
 * is unstable and may fail with J-Link auto detect, therefore adaptive clocking
 * should always be used. The adaptive clock can be select from menu:
 *  Project->Options..., section Debugger->J-Link/J-Trace  JTAG Speed - Adaptive.
 *
 ******************************************************************************/
#include <nxp/iolpc1768.h>
#include "type.h"
#include "irq.h"
#include "target.h"
#include "rtc.h"

extern volatile DWORD alarm_on;
RTCTime local_time, alarm_time, current_time;

/*****************************************************************************
**   Main Function  main()
******************************************************************************/
int main (void)
{
  TargetResetInit();

  /* Initialize RTC module */
  RTCInit();

  local_time.RTC_Sec = 0;
  local_time.RTC_Min = 0;
  local_time.RTC_Hour = 0;
  local_time.RTC_Mday = 8;
  local_time.RTC_Wday = 3;
  local_time.RTC_Yday = 12;   /* current date 07/12/2006 */
  local_time.RTC_Mon = 07;
  local_time.RTC_Year = 2006;
  RTCSetTime( local_time );   /* Set local time */

  alarm_time.RTC_Sec = 0;
  alarm_time.RTC_Min = 0;
  alarm_time.RTC_Hour = 0;
  alarm_time.RTC_Mday = 1;
  alarm_time.RTC_Wday = 0;
  alarm_time.RTC_Yday = 1;    /* alarm date 01/01/2007 */
  alarm_time.RTC_Mon = 1;
  alarm_time.RTC_Year = 2007;
  RTCSetAlarm( alarm_time );    /* set alarm time */

  /* install RTC timer handler mainly for alarm control */
  NVIC_IntEnable(NVIC_RTC);
  NVIC_IntPri(NVIC_RTC,HIGHEST_PRIORITY);

  /* mask off alarm mask, turn on IMYEAR in the counter increment interrupt
  register */
  RTCSetAlarmMask(AMRSEC|AMRMIN|AMRHOUR|AMRDOM|AMRDOW|AMRDOY|AMRMON|AMRYEAR);
  RTCCIIR = IMMIN | IMYEAR;
  /* 2007/01/01/00:00:00 is the alarm on */

  RTCStart();

  while (1)
  {         /* Loop forever */
    current_time = RTCGetTime();
    if ( alarm_on != 0 )
    {
      alarm_on = 0;
      /* Get current time when alarm is on */
      current_time = RTCGetTime();
    }
  }
  return 0;
}

/******************************************************************************
**                            End Of File
******************************************************************************/
