/*****************************************************************************
 *   ssp.h:  Header file for NXP LPC23xx/24xx Family Microprocessors
 *
 *   Copyright(C) 2006, NXP Semiconductor
 *   All rights reserved.
 *
 *   History
 *   2006.07.19  ver 1.00    Prelimnary version, first Release
 *
******************************************************************************/
#ifndef __SSP_H__
#define __SSP_H__

#define DMA_ENABLED   1

/* if USE_CS is zero, set SSEL as GPIO that you have total control of the sequence */
#define USE_CS        1

/* SPI read and write buffer size */
#define BUFSIZE       256
#define FIFOSIZE      8

/* SSP select pin */
#define SSP1_SEL      1 << 6
  
/* SSP Status register */
#define SSPSR_TFE     1 << 0
#define SSPSR_TNF     1 << 1
#define SSPSR_RNE     1 << 2
#define SSPSR_RFF     1 << 3
#define SSPSR_BSY     1 << 4

/* SSP1 CR0 register */
#define SSPCR0_DSS    1 << 0
#define SSPCR0_FRF    1 << 4
#define SSPCR0_SPO    1 << 6
#define SSPCR0_SPH    1 << 7
#define SSPCR0_SCR    1 << 8

/* SSP1 CR1 register */
#define SSPCR1_LBM    1 << 0
#define SSPCR1_SSE    1 << 1
#define SSPCR1_MS     1 << 2
#define SSPCR1_SOD    1 << 3

/* SSP1 Interrupt Mask Set/Clear register */
#define SSPIMSC_RORIM 1 << 0
#define SSPIMSC_RTIM  1 << 1
#define SSPIMSC_RXIM  1 << 2
#define SSPIMSC_TXIM  1 << 3

/* SSP1 Interrupt Status register */
#define SSPRIS_RORRIS 1 << 0
#define SSPRIS_RTRIS  1 << 1
#define SSPRIS_RXRIS  1 << 2
#define SSPRIS_TXRIS  1 << 3

/* SSP1 Masked Interrupt register */
#define SSPMIS_RORMIS 1 << 0
#define SSPMIS_RTMIS  1 << 1
#define SSPMIS_RXMIS  1 << 2
#define SSPMIS_TXMIS  1 << 3

/* SSP1 Interrupt clear register */
#define SSPICR_RORIC  1 << 0
#define SSPICR_RTIC   1 << 1

/* If RX_INTERRUPT is enabled, the SSP RX will be handled in the ISR
SSP0Receive() will not be needed. */
extern void SSP1_IRQHandler( void );
extern DWORD SSP1Init( void );
extern void SSP1Send( BYTE *Buf, DWORD Length );
extern void SSP1Receive( BYTE *buf, DWORD Length );

#endif  /* __SSP_H__ */
/*****************************************************************************
**                            End Of File
******************************************************************************/
